@@ -1,8 +1,32 @@
-/* -*- c++ -*- (enables emacs c++ mode)     */
-/* 
-   "sparse" tensors: these are not handle like sparse matrices
-   
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2000-2006 Julien Pommier
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**@file bgeot_sparse_tensors.h
+   @author  Julien Pommier <Julien.Pommier@insa-toulouse.fr>
+   @date January 2003.
+   @brief Sparse tensors, used during the assembly.
+
+   "sparse" tensors: these are not handled like sparse matrices
+   
    As an example, let say that we have a tensor t(i,j,k,l) of
    dimensions 4x2x3x3, with t(i,j,k,l!=k) == 0. 
    
@@ -20,7 +44,6 @@
 
    If we add information about the location of the non-null elements
    (by mean of strides), then we have an object of type 'tensor_ref'
-
    
    Iteration on the data of one or more tensor should be done via the
    'multi_tensor_iterator', which can iterate over common non-null
@@ -256,10 +279,12 @@ namespace bgeot {
   typedef std::vector<tensor_mask> tensor_mask_container;
 
   struct tensor_index_to_mask {
-    dim_type mask_num;
-    dim_type mask_dim;
-    tensor_index_to_mask() : mask_num(dim_type(-1)), mask_dim(dim_type(-1)) {}
-    bool is_valid() { return mask_num != dim_type(-1) && mask_dim != dim_type(-1); }
+    short_type mask_num;
+    short_type mask_dim;
+    tensor_index_to_mask() : mask_num(short_type(-1)), 
+			     mask_dim(short_type(-1)) {}
+    bool is_valid() { return mask_num != short_type(-1) && 
+	mask_dim != short_type(-1); }
   };
 
 
