@@ -1,13 +1,39 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2003-2006 Julien Pommier
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**@file getfem_assembling_tensors.h
+   @author  Julien Pommier <Julien.Pommier@insa-toulouse.fr>
+   @date January 2003.
+   @brief Generic assembly implementation.
+*/
 #ifndef GETFEM_ASSEMBLING_TENSORS_H__
 #define GETFEM_ASSEMBLING_TENSORS_H__
 
 #include <gmm_kernel.h>
 #include <getfem_mesh_fem.h>
+#include <getfem_mesh_im.h>
 #include <bgeot_sparse_tensors.h>
 #include <getfem_mat_elem_type.h>
-#include <numeric>
+#include <getfem_mat_elem.h>
 #include <map>
 
 #define ASM_THROW_PARSE_ERROR(x) DAL_THROW(std::invalid_argument, "parse error: " << x << endl << "found here:\n " << syntax_err_print());
@@ -26,12 +52,12 @@ namespace getfem {
   using bgeot::multi_tensor_iterator;
   using bgeot::TDIter;
 
-  /* 
+  class ATN_tensor;
+
+  /*
      base class for the tree built from the expression of the tensor assembly
      (ATN == Assembly Tree Node)
   */
-  class ATN_tensor;
-
   class ATN {
     std::deque< ATN_tensor* > childs_;
     std::string name_;   /* the name is a part of the parsed string */
@@ -360,7 +386,7 @@ namespace getfem {
   class asm_tokenizer {
   public:
     typedef enum { OPEN_PAR='(', CLOSE_PAR=')', COMMA=',', 
-		   SEMICOLON=';', COLON=':', EQUAL='=', MFREF='#', 
+		   SEMICOLON=';', COLON=':', EQUAL='=', MFREF='#', IMREF='%', 
 		   PLUS='+',MINUS='-', PRODUCT='.',MULTIPLY='*', 
 		   DIVIDE='/', ARGNUM_SELECTOR='$', 
 		   OPEN_BRACE='{', CLOSE_BRACE='}', 
@@ -408,22 +434,27 @@ namespace getfem {
     }
     void get_tok();
     double tok_number_dval() { assert(tok_type()==NUMBER); return curr_tok_dval; }
-    int tok_number_ival(int maxval=10000000) { 
+    int tok_number_ival(int maxval=10000000) {
       int n=int(tok_number_dval());
       if (n != curr_tok_dval) ASM_THROW_PARSE_ERROR("non an integer"); 
       if (n > maxval) ASM_THROW_PARSE_ERROR("out of bound integer");
       return n-1; /* -1 pour un indicage qui commence  1! */ 
     }
     size_type tok_mfref_num() { assert(tok_type()==MFREF); return curr_tok_ival; }
+    size_type tok_imref_num() { assert(tok_type()==IMREF); return curr_tok_ival; }
     size_type tok_argnum() { assert(tok_type()==ARGNUM_SELECTOR); return curr_tok_ival; }
   };
 
 
 
 
-  /* main class for generic assembly */
+  /** Generic assembly of vectors, matrices. 
+      
+      Many examples of use available @link asm here@endlink.
+   */
   class generic_assembly : public asm_tokenizer {
     std::deque<const mesh_fem *> mftab;/* list of the mesh_fem used in the computation */
+    std::deque<const mesh_im *> imtab;/* list of the mesh_im used in the computation */
     std::deque<pnonlinear_elem_term> innonlin;  /* alternatives to base, grad, hess in comp() for non-linear computations) */
     std::deque<base_asm_data*> indata;              /* data sources */
     std::deque<base_asm_vec*> outvec;               /* vectors in which is done the assembly */
@@ -451,10 +482,11 @@ namespace getfem {
     { set_str(s_); }
     generic_assembly(const std::string& s_,
 	       std::deque<const mesh_fem*>& mftab_, 
+	       std::deque<const mesh_im*>& imtab_, 
 	       std::deque<base_asm_data*> indata_,
 	       std::deque<base_asm_mat*> outmat_,
 	       std::deque<base_asm_vec*> outvec_) : 
-      mftab(mftab_), 
+      mftab(mftab_), imtab(imtab_),
       indata(indata_), outvec(outvec_), outmat(outmat_),
       vec_fact(0), mat_fact(0), parse_done(false)
     { set_str(s_); }    
@@ -470,32 +502,47 @@ namespace getfem {
 
     void set(const std::string& s_) { set_str(s_); }
     const std::deque<const mesh_fem*>& mf() const { return mftab; }
+    const std::deque<const mesh_im*>& im() const { return imtab; }
     const std::deque<pnonlinear_elem_term> nonlin() const { return innonlin; }
     const std::deque<base_asm_data*>& data() const { return indata; }
     const std::deque<base_asm_vec*>& vec() const { return outvec; }
     const std::deque<base_asm_mat*>& mat() const { return outmat; }
+    /// Add a new mesh_fem
     void push_mf(const mesh_fem& mf_) { mftab.push_back(&mf_); }
+    /// Add a new mesh_im
+    void push_mi(const mesh_im& im_) { imtab.push_back(&im_); }
+    /// Add a new non-linear term
     void push_nonlinear_term(pnonlinear_elem_term net) {
       innonlin.push_back(net);
     }
+    /// Add a new data (dense array)
     template< typename VEC > void push_data(const VEC& d) { 
       indata.push_back(new asm_data<VEC>(&d)); 
     }
+    /// Add a new output vector
     template< typename VEC > void push_vec(VEC& v) { 
       asm_vec<VEC> *pv = new asm_vec<VEC>(&(gmm::linalg_cast(v)));
       outvec.push_back(pv);
     }
+    /// Add a new output vector (fake const version..)
     template< typename VEC > void push_vec(const VEC& v) { 
       asm_vec<VEC> *pv = new asm_vec<VEC>(&(gmm::linalg_cast(v)));
       outvec.push_back(pv);
     }
+    /// Add a new output matrix (fake const version..)
     template< typename MAT > void push_mat(const MAT& m) { 
       outmat.push_back(new asm_mat<MAT>(&(gmm::linalg_cast(m)))); 
     }
+    /// Add a new output matrix
     template< typename MAT > void push_mat(MAT& m) { 
       outmat.push_back(new asm_mat<MAT>(&(gmm::linalg_cast(m)))); 
     }
 
+    template <typename T> void push_mat_or_vec(T &v) {
+      push_mat_or_vec(v, typename gmm::linalg_traits<T>::linalg_type());
+    }
+
+    /// used by the getfem_interface..
     void set_vec_factory(base_vec_factory *fact) { vec_fact = fact; }
     void set_mat_factory(base_mat_factory *fact) { mat_fact = fact; }
 
@@ -508,6 +555,7 @@ namespace getfem {
       t->set_name(mark_txt());
       outvars.push_back(t); return t;
     }
+    const mesh_fem& do_mf_arg_basic();
     const mesh_fem& do_mf_arg(std::vector<const mesh_fem*> *multimf = 0);
     void do_dim_spec(vdim_specif_list& lst);
     std::string do_comp_red_ops();
@@ -521,15 +569,26 @@ namespace getfem {
     tnode do_expr();
     void do_instr();
     void exec(size_type cv, dim_type face);
+    void consistency_check();
+    template <typename T> void push_mat_or_vec(T &v, gmm::abstract_vector) {
+      push_vec(v);
+    }
+    template <typename T> void push_mat_or_vec(T &v, gmm::abstract_matrix) {
+      push_mat(v);
+    }
   public:
     /* parse the string 'str' and build the tree of vtensors */
     void parse();
     /* do the assembly on the whole mesh */
-    void volumic_assembly();
+    //void volumic_assembly();
     /* do the assembly on the specified convexes */
-    void volumic_assembly(const dal::bit_vector& cvlst);
+    //void volumic_assembly(const dal::bit_vector& cvlst);
     /* do the assembly on the specified boundary */
-    void boundary_assembly(size_type boundary_number);
+    //void boundary_assembly(size_type boundary_number);
+
+    /** do the assembly on the specified region (boundary or set of convexes) */
+    void assembly(const mesh_region &region = 
+		  mesh_region::all_convexes());
   };
 }  /* end of namespace getfem.                                             */
 
