@@ -1,35 +1,30 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  GEneric Tool for Finite Element Methods (getfem)             */
-/* File    :  getfem_export.h : export and import data.                    */
-/*     									   */
-/* Date : October 15, 2001.                                                */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*          Julien Pommier, pommier@gmm.insa-tlse.fr                       */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2001-2004  Yves Renard, Julien Pommier.                   */
-/*                                                                         */
-/* This file is a part of GETFEM++                                         */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
-
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2001-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**@file getfem_export.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>, Julien Pommier <Julien.Pommier@insa-toulouse.fr>
+   @date October 15, 2001.
+   @brief Export solutions to various formats.
+*/
 #ifndef GETFEM_EXPORT_H__
 #define GETFEM_EXPORT_H__
 
@@ -43,195 +38,6 @@ namespace getfem {
   /*                                                                       */
   /* ********************************************************************* */
 
-  /** Semi-obsoleted function: In general, it is preferable to avoid this
-      function, and directly save the mesh_fem (mf.write_to_file) in one file,
-      and the fields in another file. */
-  template<typename VECT>
-    void save_solution(const std::string &filename, const mesh_fem &mf,
-		       const VECT &U, short_type K)
-  { // a corriger
-    dim_type N = mf.linked_mesh().dim();
-    dim_type P = mf.get_qdim();
-
-    std::ofstream o(filename.c_str());
-    if (!o) DAL_THROW(internal_error, "impossible to open file");
-    base_node pt1(N), pt3(P);
-    base_matrix G;
-
-    o << "% GETFEM++ DATA FILE\nBEGIN DATA ELEMENT\nN = " << int(N)
-      << "\nP = " << int(P) << "\nK = " << K << endl << endl;
-    o.precision(14);
-    for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
-      bgeot::pgeometric_trans pgt = mf.linked_mesh().trans_of_convex(cv);
-      pfem pfe = classical_fem(pgt, K);
-      pfem pf1 = mf.fem_of_element(cv);
-      // size_type nbd1 = mf.nb_dof_of_element(cv);
-      size_type nbd2 = pfe->nb_dof(cv);
-
-      o << "DIM = " << int(pgt->dim()) << endl;
-
-      if (pf1->need_G()) 
-	bgeot::vectors_to_base_matrix(G,mf.linked_mesh().points_of_convex(cv));
-
-      if (pf1->target_dim() != 1)
-	DAL_THROW(to_be_done_error, "to be done ... ");
-      fem_interpolation_context ctx(pgt,pf1,base_node(),G,cv);
-      for (size_type i = 0; i < nbd2; ++i)
-      {
-	/* point corresponding to the i th node.                           */
-	pt1 = pgt->transform(pfe->node_of_dof(cv, i),
-			     mf.linked_mesh().points_of_convex(cv));
-	for (size_type j = 0; j < N; ++j) o << pt1[j] << " ";
-	o << "  ";
-
-	/* interpolation of the solution.                                  */
-	/* faux dans le cas des lments vectoriel.                        */
-	ctx.set_xref(pfe->node_of_dof(cv, i));
-	pf1->interpolation(ctx, 
-			   gmm::sub_vector(U,
-	   gmm::sub_index(mf.ind_dof_of_element(cv))), pt3, mf.get_qdim());
-	for (size_type j = 0; j < P; ++j) o << pt3[j] << " ";
-	o << endl;
-      }
-      o << endl;
-    }
-    o << "END DATA ELEMENT" << endl;
-    o.close();
-  }
-
-  /** Semi-obsoleted function */
-  template<typename VECT>
-    void load_solution(const std::string &fi, getfem_mesh &mesh,
-		       mesh_fem &mef, VECT &U, short_type &K) {
-    std::ifstream ist(fi.c_str());
-    size_type token, DIM, Np, N;
-    char tmp[100], c;
-    bool error = false, began = false;
-    dal::dynamic_array<base_node> ptab;
-    dal::dynamic_array<scalar_type> vtab;
-    size_type nbvtab = 0;
-    dim_type P;
-    
-    if (!ist) DAL_THROW(file_not_found_error, "File " << fi << " not found");
-
-    N = 1; P = 1; K = 1; DIM = 1; // Default parameters
-    
-    if (!(ftool::read_until(ist, "BEGIN DATA ELEMENT"))) error = true;
-
-    while (!error) {
-      if (ist.eof()) { error = true; break; }
-      ftool::get_token(ist, tmp, 99); token = 0;
-      if (!strcmp(tmp, "N")) token = 1;
-      if (!strcmp(tmp, "P")) token = 2;
-      if (!strcmp(tmp, "K")) token = 3;
-      if (!strcmp(tmp, "DIM")) token = 4;
-      if (!strcmp(tmp, "END")) break;
-      if (token) {
-	if (token >= 1 && token <= 2 && began)
-	  cerr << "WARNING : Ignoring principal dimensions redefinition"<<endl;
-	else {
-	  ftool::get_token(ist, tmp, 99);
-	  if (strcmp(tmp, "=")) error = true;
-	  ftool::get_token(ist, tmp, 99);
-	  int k = atoi(tmp); if (k < 0 || k > 254) error = true;
-	  // cout << "token " << token << " val = " << k << endl;
-	  switch (token) {
-	  case 1 : DIM = N = k; if (k == 0) error = true; break;
-	  case 2 : P = k; mef.set_qdim(P); if (k == 0) error = true; break;
-	  case 3 : K = k; break;
-	  case 4 : DIM = k; break;
-	  }
-	}
-      }
-      else {
-	began = true;
-	// cout << "tmp =" << tmp << endl;
-	ist.putback(' ');
-	for(int k = strlen(tmp)-1; k >= 0; --k) ist.putback(tmp[k]);
-	size_type nbpt = 0;
-	// cout << "N = " << N << " P = " << int(P) << " K = "
-	//      << K << " DIM = " << DIM << endl;
-	while(nbpt < 1000) {
-	  if (ptab[nbpt].size() != N) ptab[nbpt].resize(N);
-	  for (dim_type i = 0; i < N; ++i) ist >> (ptab[nbpt])[i];
-	  // cout << "Ptab[" << nbpt << "] = " << ptab[nbpt] << endl;
-	  for (short_type i = 0; i < P; ++i) ist >> vtab[nbvtab * P + i];
-	  nbpt++; nbvtab++;
-	  while(ist.get(c))
-	    { if (!(isspace(c)) || c == 10) { ist.putback(c); break; } }
-	  ist.get(c); if (c != 10 && !(ist.eof())) { error = true; break; }
-	  while(ist.get(c))
-	    { if (!(isspace(c)) || c == 10) { ist.putback(c); break; } }
-	  ist.get(c); if (c == 10 || ist.eof()) break; else ist.putback(c);
-	}
-	if (nbpt >= 1000) { error = true; break; }
-	if (nbpt == 0) break;
-
-	Np = 1; for (dim_type i = 0; i < DIM; ++i) Np *= K + 1;
-	
-	if (nbpt == bgeot::alpha(DIM, K)) {
-	  std::vector<getfem::base_node> pnode(DIM+1);
-	  for (dim_type i = 0; i <= DIM; ++i) 
-	    pnode[i] = ptab[bgeot::alpha(i, K) - 1];
-	  size_type i = mesh.add_simplex_by_points(DIM, pnode.begin());
-	  mef.set_finite_element(i, getfem::PK_fem(DIM, K),
-				 getfem::exact_simplex_im(DIM));
-	}
-	else if (nbpt == Np) {
-	  std::vector<getfem::base_node> pnode(1 << DIM);
-	  size_type j = 0;
-	  for (size_type i = 0; i < (size_type(1) << DIM); ++i) {
-	    pnode[i] = ptab[j];
-	    size_type k = i + 1, l = K-1;
-	    while (!(k & 1)) { l *= K+1; k >>= 1; }
-	    j += l + 1;
-	  }
-	  j = mesh.add_parallelepiped_by_points(DIM, pnode.begin());
-	  mef.set_finite_element(j, getfem::QK_fem(DIM, K),
-				 getfem::exact_parallelepiped_im(DIM));
-	}
-	else if (nbpt == bgeot::alpha(DIM - 1, K) * bgeot::alpha(1, K)) {
-	  std::vector<getfem::base_node> pnode(2*DIM);
-	  for (dim_type i = 0; i < DIM; ++i)
-	    pnode[i] = ptab[bgeot::alpha(i, K) - 1];
-	  for (dim_type i = 0; i < DIM; ++i)
-	    pnode[i+DIM] = ptab[bgeot::alpha(i, K)-1 + bgeot::alpha(DIM-1, K)];
-	  int i = mesh.add_prism_by_points(DIM, pnode.begin());
-	  mef.set_finite_element(i, getfem::PK_prism_fem(DIM, K),
-				 getfem::exact_prism_im(DIM));
-	}
-	else 
-	  DAL_THROW(failure_error, "Unknown element in file " << fi);
-      }
-    }
-    
-    ist.close();
-    if (error) DAL_THROW(failure_error, "Format error in file " << fi);
-    
-    // Data repartition
-    //  corriger
-    
-    U.resize(mef.nb_dof());
-    std::fill(U.begin(), U.end(), 0.0);
-    std::vector<int> cp(mef.nb_dof() / P);
-    std::fill(cp.begin(), cp.end(), 0);
-    size_type l = 0;
-    for (dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i) {
-      size_type nbd = mef.nb_dof_of_element(i);
-      for (size_type j = 0; j < nbd / P; ++j, ++l) {
-	size_type dof = mef.ind_dof_of_element(i)[j*P];
-	for (short_type k = 0; k < P; ++k) U[dof +k] += vtab[l*P+k];
-	(cp[dof])++;
-      }
-    }
-    for (size_type i = 0; i < mef.nb_dof() / P; ++i) {
-      if (cp[i] == 0) DAL_THROW(internal_error, "Internal error");
-      for (short_type k = 0; k < P; ++k)
-	U[i*P +k] /= getfem::scalar_type(cp[i]);
-    }
-    
-  }
-
   inline std::string remove_spaces(const std::string &s) {
     std::string s2(s); 
     for (unsigned i=0; i < s.size(); ++i) 
@@ -239,7 +45,8 @@ namespace getfem {
     return s2;
   }
 
-  /** 
+  /** @brief VTK export.
+
       export class to VTK ( http://www.kitware.com/vtk.html ) file format 
       (not the XML format, but the old format)
 
@@ -269,7 +76,7 @@ namespace getfem {
     vtk_export(std::ostream &os_, bool ascii_ = false);
 
     /** should be called before write_*_data */
-    void exporting(const getfem_mesh& m);
+    void exporting(const mesh& m);
     void exporting(const mesh_fem& mf);
     void exporting(const stored_mesh_slice& sl);
 
@@ -303,7 +110,7 @@ namespace getfem {
         point_data (TO IMPROVE ONEDAY), if a mesh/mesh_fem is being exported,
         it is written as cell_data
     */
-    void write_mesh_quality(const getfem_mesh &m);
+    void write_mesh_quality(const mesh &m);
     void write_normals();
     const stored_mesh_slice& get_exported_slice() const;
     const mesh_fem& get_exported_mesh_fem() const;
@@ -409,23 +216,23 @@ namespace getfem {
                 << gmm::vect_size(U) << " != " << nb_val << "*" << Q);
     write_separ();
     if (Q == 1) {
-      cout << "Q=" << Q << ", " << "SCALARS " << remove_spaces(name) << " float 1\n";
+      //cout << "Q=" << Q << ", " << "SCALARS " << remove_spaces(name) << " float 1\n";
       os << "SCALARS " << remove_spaces(name) << " float 1\n";
       os << "LOOKUP_TABLE default\n";
       for (size_type i=0; i < nb_val; ++i) {
 	write_val(float(U[i]));
       }
     } else if (Q <= 3) {
-      cout << "Q=" << Q << ", " << "VECTORS " << remove_spaces(name) << " float\n";
+      //cout << "Q=" << Q << ", " << "VECTORS " << remove_spaces(name) << " float\n";
       os << "VECTORS " << remove_spaces(name) << " float\n";
       for (size_type i=0; i < nb_val; ++i) {
 	write_vec(U.begin() + i*Q);
       }
-    } else if (Q == dal::sqr(dim_)) {
+    } else if (Q == gmm::sqr(dim_)) {
       /* tensors : coef are supposed to be stored in FORTRAN order 
          in the VTK file, they are written with C (row major) order
        */
-      cout << "Q=" << Q << ", " << "TENSORS " << remove_spaces(name) << " float\n";
+      //cout << "Q=" << Q << ", " << "TENSORS " << remove_spaces(name) << " float\n";
       os << "TENSORS " << remove_spaces(name) << " float\n";
       for (size_type i=0; i < nb_val; ++i) {
         write_3x3tensor(U.begin() + i*Q);
@@ -436,7 +243,8 @@ namespace getfem {
   }
 
 
-  /** A (quite large) class for exportation of data to IBM OpenDX
+  /** @brief A (quite large) class for exportation of data to IBM OpenDX.
+
                      http://www.opendx.org/
 
       This class is more capable than the VTK export, as it is
@@ -482,7 +290,7 @@ namespace getfem {
     dx_export(const std::string& fname, bool ascii_ = false, bool append_ = false);
     dx_export(std::ostream &os_, bool ascii_ = false);
     ~dx_export(); /* the file is not complete until the destructor has been executed */
-    void exporting(const getfem_mesh& m, std::string name = std::string());
+    void exporting(const mesh& m, std::string name = std::string());
     void exporting(const mesh_fem& mf, std::string name = std::string());
     void exporting(const stored_mesh_slice& sl, bool merge_points = true, std::string name = std::string());
     /** append edges information (if you want to draw the mesh and are
@@ -493,7 +301,7 @@ namespace getfem {
        you can put whatever you want -- call this before any write_dataset or write_mesh */
     void set_header(const std::string& s);
     void write_mesh();
-    /* add an object (typically the name of a data field) to a
+    /** add an object (typically the name of a data field) to a
        'series', i.e.  an aggregate of consecutive objects. Using
        'series' is useful for animations in opendx 
 
@@ -505,9 +313,10 @@ namespace getfem {
 			  const std::string &object_name);
     void serie_add_object(const std::string &serie_name) 
     { serie_add_object(serie_name, current_data_name()); }
-    /* name of current mesh (use exporting(...) to change the current mesh) */
+    /** return the name of current mesh (use exporting(...) to change
+	the current mesh) */
     std::string current_mesh_name() { return current_mesh().name; }
-    /* name of last written data_set */
+    /** return the name of last written data_set */
     std::string current_data_name() { return current_data().name; }
     template<class VECT> void 
     write_point_data(const getfem::mesh_fem &mf, 
@@ -517,7 +326,7 @@ namespace getfem {
     /* TOBEDONE !!!!!!!!!!!
        template<class VECT> void 
     write_cell_data(const VECT& U, std::string name = std::string()); 
-    void write_mesh_quality(const getfem_mesh &m);*/
+    void write_mesh_quality(const mesh &m);*/
     void write_normals();
     const stored_mesh_slice& get_exported_slice() const;
     const mesh_fem& get_exported_mesh_fem() const;
