@@ -1,35 +1,33 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  GEneric Tool for Finite Element Methods (getfem)             */
-/* File    :  getfem_mesher : experimental mesher.                         */
-/*     									   */
-/*                                                                         */
-/* Date : May 1, 2004.                                                     */
-/* Author : Julien Pommier, Julien.Pommier@insa-toulouse.fr                */
-/*          Yves Renard, Yves.Renard@insa-toulouse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2004  Julien Pommier, Yves Renard.                        */
-/*                                                                         */
-/* This file is a part of GETFEM++                                         */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2004-2006 Julien Pommier
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
+
+/**@file getfem_mesher.h
+   @author  Julien Pommier <Julien.Pommier@insa-toulouse.fr>, Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date May 1, 2004.
+   @brief a broken mesher.
+*/
+
+#ifndef GETFEM_MESHER_H__
+#define GETFEM_MESHER_H__
 
 
 #include <getfem_mesh.h>
@@ -59,7 +57,7 @@ namespace getfem {
   // should satisfy that dist(P) is less than the euclidean distance from
   // P to the boundary and more than 1/sqrt(N) time this distance.
 
-#define SEPS 1e-5
+#define SEPS 1e-8
 
   class mesher_signed_distance : public mesher_virtual_function {
   protected:
@@ -71,6 +69,7 @@ namespace getfem {
 				   dal::bit_vector &bv) const = 0;
     virtual scalar_type grad(const base_node &P,
 			     base_small_vector &G) const = 0;
+    virtual void hess(const base_node &P, base_matrix &H) const = 0;
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const=0;
     virtual scalar_type operator()(const base_node &P) const  = 0;
@@ -81,28 +80,77 @@ namespace getfem {
   public:
     mesher_half_space(const base_node &x0_, const base_small_vector &n_)
       : x0(x0_), n(n_)
-    { n /= gmm::vect_norm2(n); xon = bgeot::vect_sp(x0, n); }
+    { n /= gmm::vect_norm2(n); xon = gmm::vect_sp(x0, n); }
     bool bounding_box(base_node &, base_node &) const
     { return false; }
     virtual scalar_type operator()(const base_node &P) const
-    { return xon - bgeot::vect_sp(P,n); }
+    { return xon - gmm::vect_sp(P,n); }
     virtual scalar_type operator()(const base_node &P,
 				   dal::bit_vector &bv) const {
-      scalar_type d = xon - bgeot::vect_sp(P,n);
-      bv[id] = (dal::abs(d) < SEPS);
+      scalar_type d = xon - gmm::vect_sp(P,n);
+      bv[id] = (gmm::abs(d) < SEPS);
       return d;
     }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const {
       id = list.size(); list.push_back(this);
     }
-
     scalar_type grad(const base_node &P, base_small_vector &G) const {
       G = n; G *= scalar_type(-1); 
-      return xon - bgeot::vect_sp(P,n);
+      return xon - gmm::vect_sp(P,n);
     }
+    void hess(const base_node &P, base_matrix &H) const {
+      gmm::resize(H, P.size(), P.size()); gmm::clear(H);
+    }
+
   };
 
+  class mesher_level_set : public mesher_signed_distance {
+    bgeot::base_poly base;
+    mutable std::vector<base_poly> gradient;
+    mutable std::vector<base_poly> hessian;
+    const fem<base_poly> *pf;
+    mutable int initialized;
+  public:
+    mesher_level_set() : initialized(0) {}
+    template <typename VECT>
+    mesher_level_set(pfem pf_, const VECT &coeff_) { init_base(pf_, coeff_); }
+    template <typename VECT> void init_base(pfem pf_, const VECT &coeff_);
+    void init_grad(void) const;
+    void init_hess(void) const;
+
+    bool bounding_box(base_node &, base_node &) const
+    { return false; }
+    virtual scalar_type operator()(const base_node &P) const
+    {  return base.eval(P.begin()); }
+    virtual scalar_type operator()(const base_node &P,
+				   dal::bit_vector &bv) const
+    { scalar_type d = (*this)(P); bv[id] = (gmm::abs(d) < SEPS); return d; }
+    virtual void register_constraints(std::vector<const
+				      mesher_signed_distance*>& list) const {
+      id = list.size(); list.push_back(this);
+    }
+    scalar_type grad(const base_node &P, base_small_vector &G) const;
+    void hess(const base_node &P, base_matrix &H) const;
+  };
+
+  template <typename VECT> 
+  void mesher_level_set::init_base(pfem pf_, const VECT &coeff_) {
+    std::vector<scalar_type> coeff(coeff_.begin(), coeff_.end());
+    if (gmm::vect_norm2(coeff) == 0)
+      DAL_THROW(dal::failure_error, "level is zero!");
+    pf = dynamic_cast<const fem<base_poly>* > (pf_.get());
+    if (!pf) DAL_THROW(dal::failure_error,
+		       "PK fem are required for level set (got "
+		       << typeid(pf_).name() << ")");
+    base = base_poly(pf->base()[0].dim(), pf->base()[0].degree());
+    for (unsigned i=0; i < pf->nb_base(0); ++i) {
+      base += pf->base()[i] * coeff[i];
+    }
+    initialized = 0; 
+  }
+
+
   class mesher_tube : public mesher_signed_distance {
     base_node x0; base_node n; scalar_type R;
   public:
@@ -119,7 +167,7 @@ namespace getfem {
     virtual scalar_type operator()(const base_node &P,
 				   dal::bit_vector &bv) const {
       scalar_type d = (*this)(P);
-      bv[id] = (dal::abs(d) < SEPS);
+      bv[id] = (gmm::abs(d) < SEPS);
       return d;
     }
     virtual void register_constraints(std::vector<const
@@ -138,6 +186,9 @@ namespace getfem {
       G /= e;
       return d;
     }
+    void hess(const base_node &, base_matrix &) const {
+      DAL_THROW(to_be_done_error, "Sorry, to be done");
+    }
   };
 
 
@@ -152,12 +203,12 @@ namespace getfem {
     }
     virtual scalar_type operator()(const base_node &P,
 				   dal::bit_vector &bv) const {
-      scalar_type d = bgeot::vect_dist2(P,x0)-R;
-      bv[id] = (dal::abs(d) < SEPS);
+      scalar_type d = gmm::vect_dist2(P,x0)-R;
+      bv[id] = (gmm::abs(d) < SEPS);
       return d;
     }
     virtual scalar_type operator()(const base_node &P) const
-    { return bgeot::vect_dist2(P,x0)-R; }
+    { return gmm::vect_dist2(P,x0)-R; }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const {
       id = list.size(); list.push_back(this);
@@ -170,6 +221,9 @@ namespace getfem {
       G /= e;
       return d;
     }
+    void hess(const base_node &, base_matrix &) const {
+      DAL_THROW(to_be_done_error, "Sorry, to be done");
+    }
   };
 
   class mesher_rectangle : public mesher_signed_distance {
@@ -217,6 +271,9 @@ namespace getfem {
       }
       return hfs[i].grad(P, G);
     }
+    void hess(const base_node &P, base_matrix &H) const {
+      gmm::resize(H, P.size(), P.size()); gmm::clear(H);
+    }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const {
       for (int k = 0; k < 2*rmin.size(); ++k)
@@ -224,11 +281,132 @@ namespace getfem {
     }
   };
 
+
+  class mesher_simplex_ref : public mesher_signed_distance {
+    // ajouter une rotation rigide, homothetie,  et translation ..
+    std::vector<mesher_half_space> hfs;
+    unsigned N;
+    base_node org;
+  public:
+    mesher_simplex_ref(unsigned N_) : N(N_) {
+      base_node no(N);
+      org = no;
+      for (unsigned k = 0; k < N; ++k) {
+	no[k] = scalar_type(1);
+	hfs.push_back(mesher_half_space(org, no));
+	no[k] = scalar_type(0);
+      }
+      std::fill(org.begin(), org.end(), scalar_type(1)/scalar_type(N));
+      no = -org;
+      hfs.push_back(mesher_half_space(org, no));
+    }
+    bool bounding_box(base_node &bmin, base_node &bmax) const {
+      bmin.resize(N); bmax.resize(N);
+      std::fill(bmin.begin(), bmin.end(), scalar_type(0));
+      std::fill(bmax.begin(), bmax.end(), scalar_type(1));
+      return true;
+    }
+    virtual scalar_type operator()(const base_node &P) const {
+      scalar_type d = - P[0];
+      for (size_type i=1; i < N; ++i) d = std::max(d, - P[i]);
+      d = std::max(d, gmm::vect_sp(P - org, org) / gmm::vect_norm2(org));
+      return d;
+    }
+
+    virtual scalar_type operator()(const base_node &P, dal::bit_vector &bv)
+      const {
+      scalar_type d = this->operator()(P);
+      if (gmm::abs(d) < SEPS) for (unsigned k = 0; k < N+1; ++k) hfs[k](P, bv);
+      return d;
+    }
+    scalar_type grad(const base_node &P, base_small_vector &G) const {
+      unsigned i = 0; scalar_type di = hfs[i](P);
+      for (unsigned k = 1; k < N+1; ++k) {
+	scalar_type dk = hfs[k](P);
+	if (dk > di) { i = k; di = dk; }
+      }
+      return hfs[i].grad(P, G);
+    }
+    void hess(const base_node &P, base_matrix &H) const {
+      gmm::resize(H, P.size(), P.size()); gmm::clear(H);
+    }
+    virtual void register_constraints(std::vector<const
+				      mesher_signed_distance*>& list) const
+    { for (unsigned k = 0; k < N+1; ++k) hfs[k].register_constraints(list); }
+  };
+
+  class mesher_prism_ref : public mesher_signed_distance {
+    // ajouter une rotation rigide, homothetie,  et translation ..
+    std::vector<mesher_half_space> hfs;
+    unsigned N;
+    base_node org;
+  public:
+    mesher_prism_ref(unsigned N_) : N(N_) {
+      base_node no(N);
+      org = no;
+      for (unsigned k = 0; k < N; ++k) {
+	no[k] = scalar_type(1);
+	hfs.push_back(mesher_half_space(org, no));
+	no[k] = scalar_type(0);
+      }
+      no[N-1] = -scalar_type(1);
+      org[N-1] = scalar_type(1);
+      hfs.push_back(mesher_half_space(org, no));
+      std::fill(org.begin(), org.end(), scalar_type(1)/scalar_type(N));
+      org[N-1] = scalar_type(0);
+      no = -org;
+      hfs.push_back(mesher_half_space(org, no));
+      
+    }
+    bool bounding_box(base_node &bmin, base_node &bmax) const {
+      bmin.resize(N); bmax.resize(N);
+      std::fill(bmin.begin(), bmin.end(), scalar_type(0));
+      std::fill(bmax.begin(), bmax.end(), scalar_type(1));
+      return true;
+    }
+    virtual scalar_type operator()(const base_node &P) const {
+      scalar_type d = - P[0];
+      for (size_type i=1; i < N; ++i) d = std::max(d, - P[i]);
+      d = std::max(d, P[N-1] - scalar_type(1));
+      d = std::max(d, gmm::vect_sp(P - org, org) / gmm::vect_norm2(org));
+      return d;
+    }
+
+    virtual scalar_type operator()(const base_node &P, dal::bit_vector &bv)
+      const {
+      scalar_type d = this->operator()(P);
+      if (gmm::abs(d) < SEPS) for (unsigned k = 0; k < N+2; ++k) hfs[k](P, bv);
+      return d;
+    }
+    scalar_type grad(const base_node &P, base_small_vector &G) const {
+      unsigned i = 0; scalar_type di = hfs[i](P);
+      for (unsigned k = 1; k < N+2; ++k) {
+	scalar_type dk = hfs[k](P);
+	if (dk > di) { i = k; di = dk; }
+      }
+      return hfs[i].grad(P, G);
+    }
+    void hess(const base_node &P, base_matrix &H) const {
+      gmm::resize(H, P.size(), P.size()); gmm::clear(H);
+    }
+    virtual void register_constraints(std::vector<const
+				      mesher_signed_distance*>& list) const
+    { for (unsigned k = 0; k < N+2; ++k) hfs[k].register_constraints(list); }
+  };
+
+
+  // Rem : It would be better for the convergence of Newton's methods to
+  //       take somthing more smooth than min. for instance tthe square root
+  //       of positive parts or negative parts depending on the position of
+  //       point (in or out the domain).
+
   extern const mesher_half_space void_signed_distance;
 
   class mesher_union : public mesher_signed_distance {
     std::vector<const mesher_signed_distance *> dists;
     mutable std::vector<scalar_type> vd;
+    mutable bool isin;
+    bool with_min;
   public:
     mesher_union(const mesher_signed_distance &a_,
 		 const mesher_signed_distance &b_,
@@ -251,7 +429,7 @@ namespace getfem {
 		 const mesher_signed_distance &s_ = void_signed_distance,
 		 const mesher_signed_distance &t_ = void_signed_distance) {
       dists.push_back(&a_); dists.push_back(&b_);
-      size_type nb = 2;
+      size_type nb = 2; with_min = true;
       if (&c_ != &void_signed_distance) { dists.push_back(&c_); ++nb; }
       if (&d_ != &void_signed_distance) { dists.push_back(&d_); ++nb; }
       if (&e_ != &void_signed_distance) { dists.push_back(&e_); ++nb; }
@@ -288,23 +466,48 @@ namespace getfem {
       return true;
     }
     virtual scalar_type operator()(const base_node &P) const {
-      scalar_type d = (*(dists[0]))(P);
-      for (size_type k = 1; k < dists.size(); ++k)
-	d = std::min(d, (*(dists[k]))(P));
+      scalar_type d, f(0), g(1);
+      if (with_min) {
+	d = (*(dists[0]))(P);
+	for (size_type k = 1; k < dists.size(); ++k)
+	  d = std::min(d, (*(dists[k]))(P));
+      }
+      else { // essai rat ...
+	isin = false;
+	for (size_type k = 0; k < dists.size(); ++k) {
+	  vd[k] = (*(dists[k]))(P);
+	  if (vd[k] <= scalar_type(0)) isin = true;
+	  f += gmm::sqr(gmm::neg(vd[k]));
+	  g *= gmm::pos(vd[k]);
+	}
+	d = isin ? -gmm::sqrt(f) : pow(g, scalar_type(1) / dists.size());
+      }
       return d;
-
     }
     scalar_type operator()(const base_node &P, dal::bit_vector &bv) const {
-      scalar_type d = vd[0] = (*(dists[0]))(P);
-      bool ok = (d > -SEPS);
-      for (size_type k = 1; k < dists.size(); ++k) {
-	vd[k] = (*(dists[k]))(P); if (vd[k] <= -SEPS) ok = false;
-	d = std::min(d,vd[k]);
+      if (with_min) {
+	scalar_type d = vd[0] = (*(dists[0]))(P);
+	bool ok = (d > -SEPS);
+	for (size_type k = 1; k < dists.size(); ++k) {
+	  vd[k] = (*(dists[k]))(P); if (vd[k] <= -SEPS) ok = false;
+	  d = std::min(d,vd[k]);
+	}
+	for (size_type k = 0; ok && k < dists.size(); ++k) {
+	  if (vd[k] < SEPS) (*(dists[k]))(P, bv);
+	}
+	return d;
       }
-      for (size_type k = 0; ok && k < dists.size(); ++k) {
-	if (vd[k] < SEPS) (*(dists[k]))(P, bv);
+      else { // essai rat ...
+	vd[0] = (*(dists[0]))(P);
+	bool ok = (vd[0] > -SEPS);
+	for (size_type k = 1; k < dists.size(); ++k) {
+	  vd[k] = (*(dists[k]))(P); if (vd[k] <= -SEPS) ok = false;
+	}
+	for (size_type k = 0; ok && k < dists.size(); ++k) {
+	  if (vd[k] < SEPS) (*(dists[k]))(P, bv);
+	}
+	return operator()(P);
       }
-      return d;
     }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const {
@@ -312,13 +515,47 @@ namespace getfem {
 	dists[k]->register_constraints(list); 
     }
     scalar_type grad(const base_node &P, base_small_vector &G) const {
+      scalar_type d;
+      if (with_min /* || gmm::abs(d) < SEPS*/) {
+	d = (*(dists[0]))(P);
+	size_type i = 0;
+	for (size_type k = 1; k < dists.size(); ++k) {
+	  scalar_type d2 = (*(dists[k]))(P);
+	  if (d2 < d) { d = d2; i = k; }
+	}
+	return dists[i]->grad(P, G);
+      }
+      else { // essai rat ...
+	d = operator()(P);
+	base_small_vector Gloc;
+	for (size_type k = 0; k < dists.size(); ++k) {
+	  dists[k]->grad(P, Gloc);
+	  if (isin) 
+	    Gloc *= -gmm::neg(vd[k]);
+	  else 
+	    Gloc *= pow(d, dists.size()) / vd[k];
+	  if (!k) G = Gloc; else G += Gloc;
+	}
+	if (isin)
+	  G *= scalar_type(1) / d;
+	else
+	  G /= pow(d, dists.size() - 1) * dists.size();
+	return d;
+      }
+    }
+    void hess(const base_node &P, base_matrix &H) const {
       scalar_type d = (*(dists[0]))(P);
-      size_type i = 0;
-      for (size_type k = 1; k < dists.size(); ++k) {
-	scalar_type d2 = (*(dists[k]))(P);
-	if (d2 < d) { d = d2; i = k; }
+      if (with_min || gmm::abs(d) < SEPS) {
+	size_type i = 0;
+	for (size_type k = 1; k < dists.size(); ++k) {
+	  scalar_type d2 = (*(dists[k]))(P);
+	  if (d2 < d) { d = d2; i = k; }
+	}
+	dists[i]->hess(P, H);
+      }
+      else {
+	DAL_THROW(to_be_done_error, "Sorry, to e done");
       }
-      return dists[i]->grad(P, G);
     }
   };
 
@@ -328,6 +565,11 @@ namespace getfem {
 
     // const mesher_signed_distance &a, &b;
   public:
+    
+    mesher_intersection(const std::vector<const mesher_signed_distance *>
+			&dists_) : dists(dists_) 
+    { vd.resize(dists.size()); }
+    
     mesher_intersection(const mesher_signed_distance &a_,
 		 const mesher_signed_distance &b_,
 		 const mesher_signed_distance &c_ = void_signed_distance,
@@ -406,8 +648,9 @@ namespace getfem {
     }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const {
-      for (size_type k = 0; k < dists.size(); ++k)
+      for (size_type k = 0; k < dists.size(); ++k) {
 	dists[k]->register_constraints(list); 
+      }
     }
     scalar_type grad(const base_node &P, base_small_vector &G) const {
       scalar_type d = (*(dists[0]))(P);
@@ -418,6 +661,15 @@ namespace getfem {
       }
       return dists[i]->grad(P, G);
     }
+    void hess(const base_node &P, base_matrix &H) const {
+      scalar_type d = (*(dists[0]))(P);
+      size_type i = 0;
+      for (size_type k = 1; k < dists.size(); ++k) {
+	scalar_type d2 = (*(dists[k]))(P);
+	if (d2 > d) { d = d2; i = k; }
+      }
+      dists[i]->hess(P, H);
+    }
   };
 
   class mesher_setminus : public mesher_signed_distance {
@@ -446,6 +698,12 @@ namespace getfem {
       if (da > db) return a.grad(P, G);
       else { b.grad(P, G); G *= scalar_type(-1); return db; }
     }
+    void hess(const base_node &P, base_matrix &H) const {
+      scalar_type da = a(P), db = -b(P);
+      if (da > db) a.hess(P, H);
+      else { b.hess(P, H); gmm::scale(H, scalar_type(-1)); }
+    }
+
   };
   
   class mesher_cylinder : public mesher_signed_distance {
@@ -474,15 +732,18 @@ namespace getfem {
     { return i2(P, bv); }
     scalar_type grad(const base_node &P, base_small_vector &G) const
       { return i2.grad(P, G); }
+    void hess(const base_node &, base_matrix &) const {
+      DAL_THROW(to_be_done_error, "Sorry, to be done");
+    }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const
     { i2.register_constraints(list); }
   };
 
 
-class mesher_ellipse : public mesher_signed_distance { // TODO
-  base_node x0; base_small_vector n, t;
-  scalar_type r, R, a;
+  class mesher_ellipse : public mesher_signed_distance { // TODO
+    base_node x0; base_small_vector n, t;
+    scalar_type r, R, a;
   public:
     mesher_ellipse(const base_node &center, const base_small_vector &no,
 		    scalar_type r_, scalar_type R_)
@@ -516,11 +777,14 @@ class mesher_ellipse : public mesher_signed_distance { // TODO
     virtual scalar_type operator()(const base_node &P,
 				   dal::bit_vector& bv) const {
       scalar_type d = this->operator()(P);
-      bv[id] = (dal::abs(d) < SEPS);
+      bv[id] = (gmm::abs(d) < SEPS);
       return d;
     }
-    scalar_type grad(const base_node &P, base_small_vector &G) const
-      { assert(0); }
+    scalar_type grad(const base_node &, base_small_vector &) const
+    { assert(0); }
+    void hess(const base_node &, base_matrix &) const {
+      DAL_THROW(to_be_done_error, "Sorry, to be done");
+    }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>& list) const
     { id = list.size(); list.push_back(this); }
@@ -545,7 +809,7 @@ class mesher_ellipse : public mesher_signed_distance { // TODO
     virtual scalar_type operator()(const base_node &P, dal::bit_vector&bv)
       const {
       scalar_type d = this->operator()(P);
-      bv[id] = (dal::abs(d) < SEPS);
+      bv[id] = (gmm::abs(d) < SEPS);
       return d;
     }
     scalar_type grad(const base_node &P, base_small_vector &G) const {
@@ -567,6 +831,9 @@ class mesher_ellipse : public mesher_signed_distance { // TODO
       }
       return d;
     }
+    void hess(const base_node &, base_matrix &) const {
+      DAL_THROW(to_be_done_error, "Sorry, to be done");
+    }
     virtual void register_constraints(std::vector<const
 				      mesher_signed_distance*>&list) const
     { id = list.size(); list.push_back(this); }
@@ -581,13 +848,25 @@ class mesher_ellipse : public mesher_signed_distance { // TODO
 
   
   // mesher
-  void build_mesh(getfem_mesh &m, const mesher_signed_distance& dist_,
+  void build_mesh(mesh &m, const mesher_signed_distance& dist_,
 		  scalar_type h0, const std::vector<base_node> &fixed_points
 		  = std::vector<base_node>(), size_type K = 1, int noise = 1,
 		  size_type iter_max = 500, int prefind = 1,
 		  scalar_type dist_point_hull = 4,
 		  scalar_type boundary_threshold_flatness = 0.11);
 
+  // exported functions
+  bool try_projection(const mesher_signed_distance& dist, base_node &X,
+		      bool on_surface = false);
+  bool pure_multi_constraint_projection
+  (const std::vector<const mesher_signed_distance*> &list_constraints,
+   base_node &X, const dal::bit_vector &cts);
+  scalar_type curvature_radius_estimate(const mesher_signed_distance &dist,
+					base_node X, bool proj = false);
+  scalar_type min_curvature_radius_estimate
+  (const std::vector<const mesher_signed_distance*> &list_constraints,
+   const base_node &X, const dal::bit_vector &cts, size_type hide_first = 0);
 
 }
 
+#endif /* GETFEM_MESHER_H__ */
