@@ -1,62 +1,39 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_solver_idgmres.h : implicitly deflated GMRES             */
-/*     									   */
-/* Date : October 6, 2003.                                                 */
-/* Authors :  Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr        */
-/*            Yves Renard, Yves.Renard@gmm.insa-tlse.fr                    */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2003  Caroline Lecalvez, Yves Renard.                     */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2003-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**@file gmm_solver_idgmres.h
+   @author  Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr, Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date October 6, 2003.
+   @brief Implicitly restarted and deflated Generalized Minimum Residual.
+*/
 #ifndef GMM_IDGMRES_H
 #define GMM_IDGMRES_H
 
 #include <gmm_kernel.h>
-#include <gmm_precond_diagonal.h>
+#include <gmm_iter.h>
 #include <gmm_dense_sylvester.h>
 
 namespace gmm {
 
-  // Implicitly restarted and deflated Generalized Minimum Residual
-  //
-  //   See: C. Le Calvez, B. Molina, Implicitly restarted and deflated
-  //        FOM and GMRES, numerical applied mathematics,
-  //        (30) 2-3 (1999) pp191-212.
-  //
-  //
-  // A : Real or complex unsymmetric matrix.
-  // x : initial guess vector and final result.
-  // b : right hand side
-  // M : preconditionner
-  // m : size of the subspace between two restarts
-  // p : number of converged ritz values seeked
-  // k : size of the remaining Krylov subspace when the p ritz values
-  //      have not yet converged 0 <= p <= k < m.
-  // tol_vp : tolerance on the ritz values.
-
-
   template <typename T> compare_vp {
     bool operator()(const std::pair<T, size_type> &a,
 		    const std::pair<T, size_type> &b) const
@@ -103,6 +80,24 @@ namespace gmm {
   }
 
 
+  /** Implicitly restarted and deflated Generalized Minimum Residual
+
+      See: C. Le Calvez, B. Molina, Implicitly restarted and deflated
+      FOM and GMRES, numerical applied mathematics,
+      (30) 2-3 (1999) pp191-212.
+      
+      @param A Real or complex unsymmetric matrix.
+      @param x initial guess vector and final result.
+      @param b right hand side
+      @param M preconditionner
+      @param m size of the subspace between two restarts
+      @param p number of converged ritz values seeked
+      @param k size of the remaining Krylov subspace when the p ritz values
+      have not yet converged 0 <= p <= k < m.
+      @param tol_vp : tolerance on the ritz values.
+      @param outer
+      @param KS
+  */
   template < typename Mat, typename Vec, typename VecB, typename Precond,
 	     typename Basis >
   void idgmres(const Mat &A, Vec &x, const VecB &b, const Precond &M,
