@@ -1,42 +1,36 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_vector.h : vectors.                                      */
-/*     									   */
-/* Date : October 13, 2002.                                                */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2002-2003 Yves Renard.                                    */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
-
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2002-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
+/**@file gmm_vector.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date October 13, 2002.
+   @brief Declaration of the vector types (gmm::rsvector , gmm::wsvector , gmm::slvector ,..)
+*/
 #ifndef GMM_VECTOR_H__
 #define GMM_VECTOR_H__
 
 #include <map>
 #include <gmm_interface.h>
 
-namespace gmm
-{
+namespace gmm {
 
   /*************************************************************************/
   /*                                                                       */
@@ -179,6 +173,9 @@ namespace gmm
   };
 
 
+  /**
+     sparse vector built upon std::map. Read and write access are quite fast (log n)
+  */
   template<typename T> class wsvector : public std::map<size_type, T> {
   public:
     
@@ -336,9 +333,20 @@ namespace gmm
 
   template<typename T> struct elt_rsvector_ {
     size_type c; T e;
-    elt_rsvector_(void) {  }
-    elt_rsvector_(size_type cc) { c = cc; }
-    elt_rsvector_(size_type cc, const T &ee) { c = cc; e = ee; }
+    /* e is initialized by default to avoid some false warnings of valgrind..
+       (from http://valgrind.org/docs/manual/mc-manual.html:
+      
+       When memory is read into the CPU's floating point registers, the
+       relevant V bits are read from memory and they are immediately
+       checked. If any are invalid, an uninitialised value error is
+       emitted. This precludes using the floating-point registers to copy
+       possibly-uninitialised memory, but simplifies Valgrind in that it
+       does not have to track the validity status of the floating-point
+       registers.
+    */
+    elt_rsvector_(void) : e(0) {}
+    elt_rsvector_(size_type cc) : c(cc), e(0) {}
+    elt_rsvector_(size_type cc, const T &ee) : c(cc), e(ee) {}
     bool operator < (const elt_rsvector_ &a) const { return c < a.c; }
     bool operator == (const elt_rsvector_ &a) const { return c == a.c; }
     bool operator != (const elt_rsvector_ &a) const { return c != a.c; }
@@ -401,6 +409,9 @@ namespace gmm
     rsvector_const_iterator(const IT &i) : it(i) {}
   };
 
+  /**
+     sparse vector built upon std::vector. Read access is fast, but insertion is O(n) 
+  */
   template<typename T> class rsvector : public std::vector<elt_rsvector_<T> > {
   public:
     
@@ -494,19 +505,20 @@ namespace gmm
     else {
       elt_rsvector_<T> ev(c, e);
       if (nb_stored() == 0) {
-	base_type_::resize(1);
-	*(this->begin()) = ev;
+	base_type_::resize(1,ev);
       }
       else {
 	iterator it = std::lower_bound(this->begin(), this->end(), ev);
 	if (it != this->end() && it->c == c) it->e = e;
 	else {
 	  size_type ind = it - this->begin();
-	  base_type_::resize(nb_stored()+1);
-	  it = this->begin() + ind;
-	  for (iterator ite = this->end() - 1; ite != it; --ite)
-	    *ite = *(ite-1);
-	  *it = ev;  //  verifier
+	  base_type_::resize(nb_stored()+1, ev);
+	  if (ind != nb_stored() - 1) {
+	    it = this->begin() + ind;
+	    for (iterator ite = this->end() - 1; ite != it; --ite)
+	      *ite = *(ite-1);
+	    *it = ev;
+	  }
 	}
       }
     }
@@ -646,7 +658,7 @@ namespace gmm
 	DAL_THROW(dimension_error,"dimensions mismatch");
 #       ifdef GMM_VERIFY
          if (same_origin(v1, v2))
-	  DAL_WARNING(2, "a conflict is possible in vector copy\n");
+	  DAL_WARNING2("a conflict is possible in vector copy\n");
 #       endif
 	copy_rsvector(v1, v2, typename linalg_traits<V>::storage_type());
     }
@@ -831,6 +843,8 @@ namespace gmm
   };
 
 
+  /** skyline vector.
+   */
   template <typename T> class slvector {
     
   public :
